﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Security.Cryptography;

namespace TelpoPush.Ability.Worker.Common
{
    public class Utils
    {

        /// <summary>
        /// 将byte[]转换成int类型（有符号整型）
        /// </summary>
        /// <param name="b">需转换的字节数组</param>
        /// <returns>转换成的整型数字</returns>
        public static int byteToInt(byte[] b, int offset, int len)//将字节数组转换成int类型
        {

            int mask = 0xff;
            int temp = 0;
            int n = 0;
            for (int i = (len - 1); i >= 0; i--)
            {
                n <<= 8;
                temp = b[offset + i] & mask;
                n |= temp;
            }
            return n;
        }

        /// <summary>
        /// 将byte[]转换成uint类型（无符号整型）
        /// </summary>
        /// <param name="b">需转换的字节数组</param>
        /// <returns>转换成的整型数字</returns>
        public static uint byteToUint(byte[] b, int offset, int len)//将字节数组转换成int类型
        {

            int mask = 0xff;
            int temp = 0;
            uint n = 0;
            for (int i = (len - 1); i >= 0; i--)
            {
                n <<= 8;
                temp = b[offset + i] & mask;
                n |= (uint)temp;
            }
            return n;
        }


        /// <summary>
        /// 对集合进行平均拆分成多个指定数量的数组 (按指定数量均分)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="Lists">集合</param>
        /// <param name="num">指定数量的数组</param>
        /// <returns></returns>
        public static List<List<T>> SpiltList<T>(List<T> Lists, int num)
        {
            List<List<T>> fz = new List<List<T>>();
            //元素数量大于等于 分组数量
            if (Lists.Count >= num)
            {
                int avg = Lists.Count / num; //每组数量
                int vga = Lists.Count % num; //余数
                for (int i = 0; i < num; i++)
                {
                    List<T> cList = new List<T>();
                    if (i + 1 == num)
                    {
                        cList = Lists.Skip(avg * i).ToList<T>();
                    }
                    else
                    {
                        cList = Lists.Skip(avg * i).Take(avg).ToList<T>();
                    }
                    fz.Add(cList);
                }
            }
            else
            {
                //最后一组 数量<=num
                fz.Add(Lists);//元素数量小于分组数量
            }
            return fz;
        }


        public static MultipartFormDataContent GetMultipartFormDataContent(Dictionary<string, object> dic, string appId, ref Dictionary<string, object> outDic)
        {
            MultipartFormDataContent mfdc = new MultipartFormDataContent();
            StringBuilder sb = new StringBuilder();
            if (dic != null && dic.Count > 0)
            {
                var dicOrderBy = dic.OrderBy(z => z.Key);
                foreach (KeyValuePair<string, object> kv in dicOrderBy)
                {
                    sb.Append($"{kv.Key}={kv.Value.ToString()}&");
                    mfdc.Add(new StringContent(kv.Value.ToString()), kv.Key);//参数, 内容在前,参数名称在后
                }
            }
            string signStr = $"{sb.ToString().Trim('&')}{appId}";
            byte[] bytes = Encoding.UTF8.GetBytes(signStr);
            byte[] hash = SHA256.Create().ComputeHash(bytes);
            StringBuilder builder = new StringBuilder();
            for (int i = 0; i < hash.Length; i++)
            {
                builder.Append(hash[i].ToString("X2"));
            }
            string sign = builder.ToString().ToLower();
            dic.Add("sign", sign);
            mfdc.Add(new StringContent(sign), "sign");//参数, 内容在前,参数名称在后
            outDic = dic;
            return mfdc;
        }
    }
}
