﻿using Confluent.Kafka;
using Dapper;
using Microsoft.AspNetCore.Mvc.ViewFeatures;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System.Net;
using System.Text;
using TelpoDataService.Util.Clients;
using TelpoDataService.Util.Entities.GpsLocationHistory;
using TelpoDataService.Util.Models;
using TelpoDataService.Util.QueryObjects;
using TelpoPush.Ability.Worker.Common;
using TelpoPush.Ability.Worker.Models.Config;
using TelpoPush.Ability.Worker.Models.Enum;
using TelpoPush.Ability.Worker.Models.MqTemplates;
using TelpoPush.Ability.Worker.Models.MqTemplates.DiBa;
using TelpoPush.Ability.Worker.Models.MqTemplates.Psych;
using TelpoPush.Ability.Worker.Models.PushTemplates;
using TelpoPush.Ability.Worker.Service.Biz;
using TelpoPush.Ability.Worker.Service.Cache;
using TelpoPush.Ability.Worker.Service.Mq;

namespace TelpoPush.Ability.Worker.Handlers
{
    public class AbilityProcess
    {
        private readonly static object _syncLocker = new object();
        private readonly IHostEnvironment _env;
        private readonly ILogger<AbilityProcess> _logger;
        private readonly HttpHelperAsync _httpHelper;
        private readonly RedisUtil _redis;
        private readonly AbilityConfig _abilityConfig;
        private readonly PsychHealthData _psychHealthData;
        private readonly GpsLocationHistoryAccessorClient<HisGpsPsych> _messagePsychClient;
        public AbilityProcess(
            IHostEnvironment env,
            ILogger<AbilityProcess> logger,
            HttpHelperAsync httpHelper,
            RedisUtil redis,
            IOptions<AbilityConfig> abilityConfig,
            PsychHealthData psychHealthData,
            GpsLocationHistoryAccessorClient<HisGpsPsych> messagePsychClient
           )
        {
            _env = env;
            _logger = logger;
            _httpHelper = httpHelper;
            _redis = redis;
            _abilityConfig = abilityConfig.Value;
            _psychHealthData = psychHealthData;
            _messagePsychClient = messagePsychClient;
        }

        public async Task SendAbility(string? message, string topic, Headers headers)
        {
            #region 数据初始验证
            bool isHandle = true;
            BaseModel model = null;
            if (!string.IsNullOrEmpty(message))
            {
                model = JsonConvert.DeserializeObject<BaseModel>(message);
                if (model == null)
                {
                    _logger.LogInformation($"[数据信息不完整] 数据解析异常：{message}");
                    isHandle = false;
                }
            }
            else
            {
                _logger.LogInformation($"[数据信息不完整] message数据异常：{message}");
                isHandle = false;
            }
            #endregion
            if (isHandle)
            {
                lock (_syncLocker)
                {
                    //Headers 解析
                    HeadersDto headersDto = new HeadersDto();
                    try
                    {
                        foreach (var item in headers)
                        {
                            if (item.Key == KafkaHeader.DataType)
                                headersDto.DataType = BitConverter.ToInt32(item.GetValueBytes(), 0);
                            else if (item.Key == KafkaHeader.AlarmType)
                                headersDto.AlarmType = BitConverter.ToInt32(item.GetValueBytes(), 0);
                            else if (item.Key == KafkaHeader.OperType)
                                headersDto.OperType = BitConverter.ToInt32(item.GetValueBytes(), 0);
                        }
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError($"当前工作线程Headers异常,{ex.Message}|{ex.Source}|{ex.StackTrace}");
                    }
                    try
                    {
                        #region  注释
                        //string dataType = headersDto.DataType != null ? "_" + headersDto.DataType : "";
                        //string alarmType = headersDto.AlarmType != null ? "_" + headersDto.AlarmType : "";
                        //string operType = headersDto.OperType != null ? "_" + headersDto.OperType : "";
                        //string key = dataType + alarmType + operType;
                        //var dataStatus = _redis.IsDateStatus(model, imei, key).Result;
                        //过滤
                        //if (headersDto.DataType == (int)MqDataType.TemperatureInfo
                        //    || headersDto.DataType == (int)MqDataType.Temperature1Info
                        //    || headersDto.DataType == (int)MqDataType.BindDevice
                        //    || headersDto.DataType == (int)MqDataType.AbilityInfo
                        //    || headersDto.DataType == (int)MqDataType.HeartRateInfo
                        //    || headersDto.DataType == (int)MqDataType.HeartRate1Info
                        //    || headersDto.DataType == (int)MqDataType.Spo2Info
                        //    || headersDto.DataType == (int)MqDataType.Spo21Info
                        //    || headersDto.DataType == (int)MqDataType.BloodPressInfo
                        //    || headersDto.DataType == (int)MqDataType.BloodPress1Info
                        //    || headersDto.DataType == (int)MqDataType.SportResult
                        //    || headersDto.DataType == (int)MqDataType.BloodSugar
                        //    )
                        //    dataStatus.isPush = true;
                        //dataStatus.isPush = true;
                        //if (dataStatus.isPush)
                        //{
                        //switch (topic)
                        //{
                        //    case "topic.push.third":
                        //        switch (headersDto.DataType)
                        //        {
                        //            case (int)MqDataType.AlarmInfo:  //报警消息
                        //                break;
                        //            case (int)MqDataType.TemperatureInfo: //体温消息
                        //                break;
                        //            case (int)MqDataType.AbilityInfo: //定位消息
                        //               // DataServicePusAbility(model, imei);
                        //                break;
                        //            case (int)MqDataType.StepInfo: //步数消息

                        //                break;
                        //            case (int)MqDataType.BatteryLevelInfo:  //电量消息
                        //                break;
                        //            case (int)MqDataType.DeviceCallLog:  //设备通话记录

                        //                break;
                        //            case (int)MqDataType.DeviceSmsLog:  //设备短信记录

                        //                break;
                        //            case (int)MqDataType.DeviceConfigInfo:  //设备配置信息

                        //                break;
                        //            case (int)MqDataType.Status: //设备状态（offline,online）

                        //                break;
                        //            case (int)MqDataType.Active:  //设备激活状态
                        //                break;
                        //            case (int)MqDataType.reply:  //指令回调

                        //                break;
                        //            case (int)MqDataType.Weather:  //天气查询

                        //                break;
                        //            case (int)MqDataType.ReadMsg:  //短消息阅读

                        //                break;
                        //            case (int)MqDataType.StudyAINotifyStatusUpload:  //学习能力状态

                        //                break;
                        //            case (int)MqDataType.HeartRateInfo:  //心率

                        //                break;
                        //            case (int)MqDataType.HeartRate1Info:  //周期性心率

                        //                break;
                        //            case (int)MqDataType.Spo2Info:  //血氧

                        //                break;
                        //            case (int)MqDataType.Spo21Info:  //周期性血氧

                        //                break;
                        //            case (int)MqDataType.Temperature1Info:  //周期性报体温数据

                        //                break;
                        //            case (int)MqDataType.DrownReportInfo:  //防溺水告警 

                        //                break;
                        //            case (int)MqDataType.WearStatusInfo:  //手表未佩戴告警 

                        //                break;
                        //            case (int)MqDataType.BloodPressInfo:  //血压 

                        //                break;
                        //            case (int)MqDataType.BloodPress1Info:  //周期性血压 

                        //                break;
                        //            case (int)MqDataType.PsychInfo:  //心理监测

                        //                break;
                        //            case (int)MqDataType.AiCallResult:  //AI呼叫结果回调

                        //            case (int)MqDataType.CrossBorder:  //越界上报(围栏进出告警)

                        //                break;
                        //            case (int)MqDataType.SportResult:  //运动数据上报

                        //                break;
                        //            case (int)MqDataType.BindDevice:  //绑定业务

                        //                break;
                        //            case (int)MqDataType.BloodSugar:  //血糖业务

                        //                break;
                        //            default:
                        //                break;
                        //        }
                        //        break;
                        //    default:
                        //        break;
                        //}
                        //}
                        //else
                        //    _logger.LogInformation($"数据未处理(历史数据):{JsonConvert.SerializeObject(dataStatus)}");

                        #endregion

                        switch (topic)
                        {
                            case "topic.push.ability":
                                switch (headersDto.DataType)
                                {
                                    case (int)MqDataType.StudyAINotifyStatusUpload: //学习能力状态
                                        StudyAINotifyStatusUploadPush(model);
                                        break;
                                    default:
                                        break;
                                }
                                break;
                            default:
                                break;
                        }

                    }
                    catch (Exception ex)
                    {
                        _logger.LogError($"当前工作线程异常: {ex.Message}|{ex.Source}|{ex.StackTrace}");
                    }
                }
            }
        }

        //位置
        #region 设备学习能力状态事件推送（第三方）
        public async Task StudyAINotifyStatusUploadPush(BaseModel model)
        {
            var obj = JsonConvert.DeserializeObject<MqStudyAINotifyStatusTemplate>(model.data.ToString());

          var  device =_redis.GetGpsDevice(obj.imei);
            if (device != null)
            {
                string imei = obj.imei;
                try
                {
                    string manufactorId = _abilityConfig.RzlManufactorId;
                    var manufactor = await _redis.GetManufactorImei(imei);
                    if (manufactor != null && !string.IsNullOrEmpty(obj.agencyId))
                    {
                        if (manufactor.manufactorId == obj.agencyId)
                            manufactorId = obj.agencyId;
                        else
                            manufactorId = manufactor.manufactorId;
                    }
                    string[] abilityNameArr = { "psych", "psych_bian", "fence" };

                    if (obj?.abilityName.ToLower() == "psych")
                    {

                        var isBianPsych = await _redis.GetPsychImei(imei);
                        if (isBianPsych > 0)
                            obj.abilityName = "psych_bian";
                    }

                    // var urlInfo = await _redisBaseManager.GetManufactorCallbackUrl(obj.agencyId);
                    switch (obj?.abilityName.ToLower())
                    {
                        case "study_ai":

                            break;
                        case "homework":

                            break;
                        case "base":
                            break;
                        case "diba":
                            #region diba
                            ActionDibaStudy remarkObj = null;
                            if (!string.IsNullOrEmpty(obj.remark))
                            {
                                remarkObj = JsonConvert.DeserializeObject<ActionDibaStudy>(obj.remark);
                                remarkObj.imei = imei;
                            }
                            switch (obj.action.ToLower())
                            {
                                case "action_download": //下载
                                    remarkObj.@event = "download";
                                    break;
                                case "action_start": //开始
                                    remarkObj.@event = "start";
                                    remarkObj.progress = 0;
                                    #region 原文
                                    // {
                                    // "abilityname": "diba",
                                    //	"messageid": "1678241606000",
                                    //	"imei": "862622051637344",
                                    //	"action": "action_start",
                                    //	"agencyid": "",
                                    //	"remark": "{\"fileId\":\"4307\",\"timestamp\":1678241606}",
                                    //	"status": "",
                                    //	"timestamp": "1678270406000"
                                    //}
                                    #endregion
                                    break;
                                case "action_stop": //停止
                                    remarkObj.@event = "stop";
                                    #region 原文
                                    // {
                                    // "abilityname": "diba",
                                    //	"messageid": "1678256130000",
                                    //	"imei": "862622051637344",
                                    //	"action": "action_stop",
                                    //	"agencyid": "",
                                    //	"remark": "{\"fileId\":\"4307\",\"progress\":\"19\",\"timestamp\":1678256130}",
                                    //	"status": "",
                                    //	"timestamp": "1678284930000"
                                    //}
                                    #endregion
                                    break;
                                case "action_pause": //暂定
                                    remarkObj.@event = "pause";
                                    #region 原文
                                    // {
                                    // "abilityname": "diba",
                                    //	"messageid": "1678256089000",
                                    //	"imei": "862622051637344",
                                    //	"action": "action_pause",
                                    //	"agencyid": "",
                                    //	"remark": "{\"fileId\":\"4307\",\"progress\":\"12\",\"timestamp\":1678256089}",
                                    //	"status": "",
                                    //	"timestamp": "1678284889000"
                                    //}
                                    #endregion
                                    break;
                                case "action_resume": //恢复
                                    remarkObj.@event = "resume";
                                    #region 原文
                                    // {
                                    //   "abilityname": "diba",
                                    //	"messageid": "1678256107000",
                                    //	"imei": "862622051637344",
                                    //	"action": "action_resume",
                                    //	"agencyid": "",
                                    //	"remark": "{\"fileId\":\"4307\",\"progress\":\"12\",\"timestamp\":1678256107}",
                                    //	"status": "",
                                    //	"timestamp": "1678284907000"
                                    //}
                                    #endregion
                                    break;
                            }
                            string dibaPushData = "";
                            if (remarkObj != null)
                                dibaPushData = JsonConvert.SerializeObject(remarkObj);
                            if (!string.IsNullOrEmpty(dibaPushData))
                            {
                                string tokenUrl = string.Format("{0}/client/api/getAccessToken", _abilityConfig.DiBaUrl);
                                var token = new { id = _abilityConfig.DiBaAppId, key = _abilityConfig.DiBaKey };

                                string responseDibaToken = await _httpHelper.PostAsync(tokenUrl, JsonConvert.SerializeObject(token));
                                _logger.LogInformation($"[迪爸精灵事(token)<{obj.remark}>]请求参数:{JsonConvert.SerializeObject(token)},URL: {tokenUrl}: 返回结果:{responseDibaToken}");
                                if (responseDibaToken.Contains("code") && responseDibaToken.Contains("data"))
                                {
                                    var tokenObj = JsonConvert.DeserializeObject<DiBaTokenModel>(responseDibaToken);
                                    if (!string.IsNullOrEmpty(tokenObj.data))
                                    {
                                        string reqquestUrl = string.Format("{0}/client/api/studyRecord/updateEvent", _abilityConfig.DiBaUrl);
                                        Dictionary<string, string> headers = new Dictionary<string, string> { { "AccessToken", tokenObj.data } };
                                        var responseDibaReqquest = await _httpHelper.PostAsync(reqquestUrl, dibaPushData, headers);
                                        _logger.LogInformation($"[迪爸精灵事(推送)<{obj.remark}>]URL:{reqquestUrl}: 请求参数:{dibaPushData},返回结果:{responseDibaReqquest}");
                                    }
                                }
                            }
                            #endregion
                            break;
                        case "psych":
                            #region psych
                            if (obj.status.ToLower() == "physicalhelath_upload_success")
                            {
                                ActionPsychStudy psycRemarkObj = null;
                                if (!string.IsNullOrEmpty(obj.remark))
                                {
                                    try
                                    {
                                        psycRemarkObj = JsonConvert.DeserializeObject<ActionPsychStudy>(obj.remark);
                                        psycRemarkObj.imei = imei;
                                        List<PsychMeasure> measures = new List<PsychMeasure>();
                                        List<PsychTemperature> temperatures = new List<PsychTemperature>();
                                        List<PsychPPG> ppg = new List<PsychPPG>();
                                        List<PsychMotion> acc = new List<PsychMotion>();
                                        List<PsychHealthDataInfo> healthData = new List<PsychHealthDataInfo>();
                                        string lbs = "";
                                        int stepTotal = 0;
                                        StringBuilder sb = new StringBuilder();
                                        var folder = $"{psycRemarkObj.imei}/{psycRemarkObj.fileName.Replace(".dat", "")}";
                                        string objectName = $"{folder}/{psycRemarkObj.fileName}";
                                        string fileUrl = $"https://aissjl-psychological.oss-cn-beijing.aliyuncs.com/{objectName}";
                                        WebClient mywebclient = new WebClient();
                                        byte[] bt = mywebclient.DownloadData(fileUrl);
                                        long start_timestamp = 0;
                                        if (bt.Length > 0)
                                        {
                                            #region 数据解析
                                            int offsetLen = 4;
                                            int offset = 0;
                                            //采集时间
                                            sb.Append($"采集时间\r\n");
                                            int t = Utils.byteToInt(bt, offset, 4);
                                            sb.Append($"{t}\r\n");
                                            start_timestamp = (long)t * 1000;
                                            offset = 4;
                                            //心率
                                            sb.Append($"心率\r\n");
                                            int heartRateCnt = Utils.byteToInt(bt, offset, 4);
                                            for (int i = 0; i < heartRateCnt; i++)
                                            {
                                                uint heartRateTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                                int heartRateValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                                long timestamp = (long)heartRateTimestamp * 1000;
                                                measures.Add(new PsychMeasure { timestamp = timestamp, type = 4, value = heartRateValue });
                                                healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 4, value = heartRateValue });
                                                sb.Append($"{heartRateTimestamp}|{heartRateValue}\r\n");
                                            }
                                            //血氧
                                            sb.Append($"血氧\r\n");
                                            offset += 28;
                                            int spoCnt = Utils.byteToInt(bt, offset, 4);
                                            for (int i = 0; i < spoCnt; i++)
                                            {
                                                uint spoTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                                int spoValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                                long timestamp = (long)spoTimestamp * 1000;
                                                measures.Add(new PsychMeasure { timestamp = timestamp, type = 3, value = spoValue });
                                                healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 3, value = spoValue });
                                                sb.Append($"{spoTimestamp}|{spoValue}\r\n");
                                            }
                                            //血压
                                            sb.Append($"血压\r\n");
                                            offset += 28;
                                            int bloodCnt = Utils.byteToInt(bt, offset, 4);
                                            for (int i = 0; i < bloodCnt; i++)
                                            {
                                                uint bloodTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 12)), 4);
                                                int bloodSbp = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 12)), 4);  //收缩压
                                                int bloodDbp = Utils.byteToInt(bt, (offset + offsetLen + 8 + (i * 12)), 4);//舒张压
                                                long timestamp = (long)bloodTimestamp * 1000;
                                                measures.Add(new PsychMeasure { timestamp = timestamp, type = 1, value = bloodDbp });
                                                measures.Add(new PsychMeasure { timestamp = timestamp, type = 2, value = bloodSbp });
                                                healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 12, value = bloodSbp, value2 = bloodDbp });
                                                sb.Append($"{bloodTimestamp}|{bloodSbp}|{bloodDbp}\r\n");
                                            }
                                            //体温
                                            sb.Append($"体温\r\n");
                                            offset += 40;
                                            int tempCnt = Utils.byteToInt(bt, offset, 4);
                                            for (int i = 0; i < tempCnt; i++)
                                            {
                                                uint tempTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                                int tempValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                                long timestamp = (long)tempTimestamp * 1000;
                                                double value = (double)tempValue / 10;
                                                temperatures.Add(new PsychTemperature { timestamp = timestamp, value = value });
                                                healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 6, value = tempValue });
                                                sb.Append($"{tempTimestamp}|{tempValue}\r\n");
                                            }
                                            //步数
                                            sb.Append($"步数\r\n");
                                            offset += 28;
                                            int stepCnt = Utils.byteToInt(bt, offset, 4);//128
                                            for (int i = 0; i < stepCnt; i++)
                                            {
                                                uint stepTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                                int stepValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                                long timestamp = (long)stepTimestamp * 1000;
                                                measures.Add(new PsychMeasure { timestamp = timestamp, type = 5, value = stepValue });
                                                healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 5, value = stepValue });
                                                sb.Append($"{stepTimestamp}|{stepValue}\r\n");
                                                stepTotal += stepValue;
                                            }

                                            //ppg
                                            sb.Append($"ppg\r\n");
                                            offset += 52;
                                            int ppgCnt = Utils.byteToInt(bt, offset, 4);//180
                                            for (int i = 0; i < ppgCnt; i++)
                                            {
                                                List<int> vals = new List<int>();
                                                uint ppgTimestamp = 0;
                                                StringBuilder sbppg = new StringBuilder();
                                                for (int j = 0; j < 21; j++)
                                                {
                                                    int set = offset + offsetLen * (j + 1) + (i * 84);
                                                    if (j == 0)
                                                    {
                                                        uint val = Utils.byteToUint(bt, set, 4);
                                                        ppgTimestamp = Utils.byteToUint(bt, set, 4);
                                                        sbppg.Append($"{val},");
                                                    }
                                                    else
                                                    {
                                                        int val = Utils.byteToInt(bt, set, 4);
                                                        int new_val = val + 4096;
                                                        vals.Add(new_val);
                                                        sbppg.Append($"{new_val},");
                                                        //vals.Add(val);
                                                        //sbppg.Append($"{val},");
                                                    }
                                                }
                                                long timestamp = (long)ppgTimestamp * 1000;
                                                ppg.Add(new PsychPPG { timestamp = timestamp, value = vals });
                                                sb.Append($"({i + 1}){sbppg.ToString()}\r\n");
                                            }
                                            //acc
                                            sb.Append($"acc\r\n");
                                            offset += 30244;
                                            int accCnt = Utils.byteToInt(bt, offset, 4);//180
                                            for (int i = 0; i < accCnt; i++)
                                            {
                                                List<double> vals = new List<double>();
                                                uint accTimestamp = 0;
                                                StringBuilder sbacc = new StringBuilder();
                                                for (int j = 0; j < 31; j++)
                                                {
                                                    int set = offset + offsetLen * (j + 1) + (i * 124);
                                                    if (j == 0)
                                                    {
                                                        uint val = Utils.byteToUint(bt, set, 4);
                                                        accTimestamp = val;
                                                        sbacc.Append($"{val},");
                                                    }
                                                    else
                                                    {
                                                        int val = Utils.byteToInt(bt, set, 4);
                                                        double avg = Math.Round((double)val / 1000, 4);
                                                        vals.Add(avg);
                                                        sbacc.Append($"{avg},");
                                                    }
                                                }
                                                List<List<double>> value = Utils.SpiltList<double>(vals, 10);
                                                long timestamp = (long)accTimestamp * 1000;
                                                string valueCsv = "";
                                                string valueCsv1 = "";
                                                for (int x = 0; x < 10; x++)
                                                {
                                                    if (x == 0)
                                                        valueCsv1 = string.Join(",", value[x]);
                                                    else
                                                        valueCsv += $"[0,0,0,{string.Join(",", value[x])}],";
                                                }
                                                string valueCsvStr = $"{timestamp}|[[0,0,0,{valueCsv1}],{valueCsv.Trim(',')}]";
                                                acc.Add(new PsychMotion { timestamp = timestamp, value = value, valueCsvStr = valueCsvStr });
                                                sb.Append($"({i + 1}){sbacc.ToString()}\r\n");
                                            }

                                            if (bt.Length == 67884)
                                            {
                                                sb.Append($"lbs\r\n"); //Lbs数据ascII码数据(256字符)
                                                offset += 37204;
                                                byte[] result = bt.Skip(offset).Take(256).ToArray();
                                                string lbsValue = Convert.ToString(Encoding.ASCII.GetString(result));//str为ASCII码对应的字符
                                                lbsValue = lbsValue.Replace("\0", ""); //去掉空格
                                                sb.Append($"{lbsValue}\r\n");
                                                lbs = lbsValue;
                                            }
                                            #endregion
                                        }
                                        var dataInfo = new { timestamp = start_timestamp, measures, temperatures, ppgs = ppg, motions = acc, lbs };
                                        var psycJson = new { psycRemarkObj.imei, psycRemarkObj.fileName, data = dataInfo };
                                        string psycJsonStr = JsonConvert.SerializeObject(psycJson);
                                        //数据采集解析保存
                                        string timeStamp = psycRemarkObj.fileName.Replace(".dat", "");

                                        try
                                        {
                                            var param = new GeneralParam
                                            {
                                                Filters = new List<QueryFilterCondition>{
                                      new QueryFilterCondition {
                                            Key=nameof(HisGpsPsych.Imei),
                                            Value=imei,
                                            ValueType=QueryValueTypeEnum.String,
                                            Operator=QueryOperatorEnum.Equal
                                         },
                                     new QueryFilterCondition {
                                            Key=nameof(HisGpsPsych.TimeStamp),
                                            Value=timeStamp,
                                            ValueType=QueryValueTypeEnum.Int,
                                            Operator=QueryOperatorEnum.Equal
                                         }
                                    }
                                            };
                                            var hisGpsPsych = _messagePsychClient.GetFirst(param);
                                            if (hisGpsPsych != null)
                                            {
                                                hisGpsPsych.FileDatData = sb.ToString();
                                                hisGpsPsych.FileDatDataTime = DateTime.Parse(model.time);
                                                if (stepTotal <= 0)
                                                    hisGpsPsych.ApiResult = $"解析后步数为{stepTotal},上传OSS，不推送第三方,";
                                               await _messagePsychClient.UpdateAsync(hisGpsPsych);
                                                _logger.LogError($"[心理数据解析结果保存<{obj.remark}>] 成功");
                                            }
                                        }
                                        catch (Exception ex2)
                                        {
                                            _logger.LogError($"[心理数据解析结果保存<<{obj.remark}>] 异常, {ex2.Message}|{ex2.Source}|{ex2.StackTrace}");
                                        }

                                        if (start_timestamp > 0)
                                        {
                                            await _psychHealthData.Save(imei, obj.msgId, healthData,"");//健康数据保存
                                            //if (stepTotal > 0) //没有步数信息不推送
                                            //{
                                                string psycReqquestUrl = $"{_abilityConfig.CoreServiceApiUrl}/OpenPsych/terminal/fileuploaresult";
                                                var psycResponseStr = await _httpHelper.PostAsync(psycReqquestUrl, psycJsonStr);
                                                JObject psycJsons = JObject.FromObject(psycJson);
                                                psycJsons.Remove("data");
                                                _logger.LogInformation($"[心理数据文件(推送)第1次<{obj.remark}>]URL:{psycReqquestUrl}: 请求参数:{JsonConvert.SerializeObject(psycJsons)},返回结果:{psycResponseStr}");
                                                if (psycResponseStr.Contains("请求异常"))
                                                {
                                                    psycResponseStr = await _httpHelper.PostAsync(psycReqquestUrl, psycJsonStr);//重新推送
                                                    _logger.LogInformation($"[心理数据文件(推送异常)第2次<{obj.remark}>]URL:{psycReqquestUrl}: 请求参数:{JsonConvert.SerializeObject(psycJsons)},返回结果:{psycResponseStr}");
                                                }
                                            //}
                                            //else
                                            //    _logger.LogInformation($"[心理数据文件(解析<{obj.remark}>]:{fileUrl}解析异常,步数为：{stepTotal}");
                                        }
                                        else
                                            _logger.LogInformation($"[心理数据文件(解析)<{obj.remark}>]:{fileUrl} 解析异常，采集时间戳：{start_timestamp}");
                                    }
                                    catch (Exception ex)
                                    {
                                        if (model.messageId.Contains("rep_psych"))
                                            _logger.LogError($"[(第2次解析)心理数据异常<{obj.remark}>] 事件信息:{obj.remark},异常,{ex.Message}|{ex.Source}|{ex.StackTrace}");
                                        else
                                        {
                                            _logger.LogError($"[(第1次解析)心理数据异常<{obj.remark}>] 事件信息:{obj.remark},异常,{ex.Message}|{ex.Source}|{ex.StackTrace}");
                                            DateTime dt = DateTime.Now;
                                            model.time = dt.ToString("yyyy-MM-dd HH:mm:ss");
                                            model.messageId = $"rep_psych_{model.messageId}";
                                            await _psychHealthData.Push(imei, MqDataType.StudyAINotifyStatusUpload, model);//重试
                                            _logger.LogError($"[心理数据异常<{obj.remark}>] 异常,重试");
                                        }
                                    }
                                }
                            }
                            else
                            {
                                //重新开通
                                var psychConfig = await _redis.GetPsychConfig(imei);
                                if (psychConfig != null)
                                {
                                    psychConfig.abilityName = "psych";
                                    Dictionary<string, string> headers = new Dictionary<string, string>();
                                    headers.Add("AuthKey", "key1");
                                    string psychUrl = $"{_abilityConfig.GpsWebApiUrl}/api/Command/SetPsychAbilityConfig";
                                    string psychJsonStr = JsonConvert.SerializeObject(psychConfig);
                                    var psychResult = await _httpHelper.PostAsync(psychUrl, psychJsonStr, headers);
                                    _logger.LogInformation($"[心理监测异常事件重新下发能力开通<{imei}>] url:{psychUrl},参数:{psychJsonStr},结果：{psychResult}");
                                }
                            }
                            #endregion
                            break;
                        case "psych_bian":
                            #region psych_bian
                            if (!string.IsNullOrEmpty(obj.remark))
                            {
                                try
                                {
                                    ActionPsychStudy psycBianRemarkObj = JsonConvert.DeserializeObject<ActionPsychStudy>(obj.remark);
                                    psycBianRemarkObj.imei = imei;
                                    //string version = "";
                                    //int start_timestamp = 0;
                                    List<BiAnRRI> rri = new List<BiAnRRI>();

                                    List<PsychMeasure> measures = new List<PsychMeasure>();
                                    List<PsychTemperature> temperatures = new List<PsychTemperature>();
                                    List<PsychPPG> ppg = new List<PsychPPG>();
                                    List<PsychMotion> acc = new List<PsychMotion>();
                                    List<PsychHealthDataInfo> healthData = new List<PsychHealthDataInfo>();
                                    List<BiAnPPG> bianPPG = new List<BiAnPPG>();

                                    string lbs = "";
                                    int start_timestamp = 0;
                                    StringBuilder sb = new StringBuilder();
                                    var folder = $"{psycBianRemarkObj.imei}/{psycBianRemarkObj.fileName.Replace(".dat", "")}";
                                    string objectName = $"{folder}/{psycBianRemarkObj.fileName}";
                                    string fileUrl = $"https://aissjl-psychological.oss-cn-beijing.aliyuncs.com/{objectName}";
                                    WebClient mywebclient = new WebClient();
                                    byte[] bt = mywebclient.DownloadData(fileUrl);
                                    if (bt.Length > 0)
                                    {
                                        #region  rri 数据解析
                                        /*                          int offsetLen = 4;
                                                                  int offset = 0;
                                                                  //版本
                                                                  sb.Append($"版本信息\r\n"); //0
                                                                  byte[] result = bt.Skip(offset).Take(16).ToArray();
                                                                  string versionValue = Convert.ToString(Encoding.ASCII.GetString(result));//str为ASCII码对应的字符
                                                                  versionValue = versionValue.Replace("\0", ""); //去掉空格
                                                                  version = versionValue;
                                                                  sb.Append($"{versionValue}\r\n");
                                                                  sb.Append($"采集时间\r\n");
                                                                  offset += 16;
                                                                  int t = Utils.byteToInt(bt, offset, 4);
                                                                  start_timestamp = t;
                                                                  sb.Append($"{t}\r\n");
                                                                  sb.Append($"rri条数\r\n");
                                                                  offset += 4;
                                                                  int line_count = Utils.byteToInt(bt, offset, 4);
                                                                  sb.Append($"{line_count}\r\n");
                                                                  sb.Append($"rri\r\n");
                                                                  for (int i = 0; i < line_count; i++)
                                                                  {
                                                                      offset += offsetLen;
                                                                      int t_rri = Utils.byteToInt(bt, offset, 4);
                                                                      offset += offsetLen;
                                                                      int rri_count = Utils.byteToInt(bt, offset, offsetLen);
                                                                      string rriStr = "";
                                                                      List<int> rri_val = new List<int>();
                                                                      for (int j = 0; j < rri_count; j++)
                                                                      {
                                                                          int set = (offset + offsetLen + (j * offsetLen));
                                                                          int val = Utils.byteToInt(bt, set, offsetLen);
                                                                          rriStr += $"{val},";
                                                                          rri_val.Add(val);
                                                                      }
                                                                      sb.Append($"({i}){t_rri},{rri_count},[{rriStr.Trim(',')}]\r\n");
                                                                      rri.Add(new BiAnRRI() { rri = rri_val, time = t_rri, uid = $"{imei}" });
                                                                      offset += (rri_count * offsetLen);
                                                                  }*/
                                        #endregion

                                        #region ppg 数据解析
                                        int offsetLen = 4;
                                        int offset = 0;
                                        //采集时间
                                        sb.Append($"彼岸采集时间\r\n");
                                        int t = Utils.byteToInt(bt, offset, 4);
                                        sb.Append($"{t}\r\n");
                                        start_timestamp = t;
                                        offset = 4;
                                        //心率
                                        sb.Append($"彼岸心率\r\n");
                                        int heartRateCnt = Utils.byteToInt(bt, offset, 4);
                                        for (int i = 0; i < heartRateCnt; i++)
                                        {
                                            uint heartRateTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                            int heartRateValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                            int timestamp = (int)heartRateTimestamp;
                                            measures.Add(new PsychMeasure { timestamp = timestamp, type = 4, value = heartRateValue });
                                            healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 4, value = heartRateValue });
                                            sb.Append($"{heartRateTimestamp}|{heartRateValue}\r\n");
                                        }
                                        //血氧
                                        sb.Append($"彼岸血氧\r\n");
                                        offset += 28;
                                        int spoCnt = Utils.byteToInt(bt, offset, 4);
                                        for (int i = 0; i < spoCnt; i++)
                                        {
                                            uint spoTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                            int spoValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                            int timestamp = (int)spoTimestamp;
                                            measures.Add(new PsychMeasure { timestamp = timestamp, type = 3, value = spoValue });
                                            healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 3, value = spoValue });
                                            sb.Append($"{spoTimestamp}|{spoValue}\r\n");
                                        }
                                        //血压
                                        sb.Append($"彼岸血压\r\n");
                                        offset += 28;
                                        int bloodCnt = Utils.byteToInt(bt, offset, 4);
                                        for (int i = 0; i < bloodCnt; i++)
                                        {
                                            uint bloodTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 12)), 4);
                                            int bloodSbp = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 12)), 4);  //收缩压
                                            int bloodDbp = Utils.byteToInt(bt, (offset + offsetLen + 8 + (i * 12)), 4);//舒张压
                                            int timestamp = (int)bloodTimestamp;
                                            measures.Add(new PsychMeasure { timestamp = timestamp, type = 1, value = bloodDbp });
                                            measures.Add(new PsychMeasure { timestamp = timestamp, type = 2, value = bloodSbp });
                                            healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 12, value = bloodSbp, value2 = bloodDbp });
                                            sb.Append($"{bloodTimestamp}|{bloodSbp}|{bloodDbp}\r\n");
                                        }
                                        //体温
                                        sb.Append($"彼岸体温\r\n");
                                        offset += 40;
                                        int tempCnt = Utils.byteToInt(bt, offset, 4);
                                        for (int i = 0; i < tempCnt; i++)
                                        {
                                            uint tempTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                            int tempValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                            int timestamp = (int)tempTimestamp;
                                            double value = (double)tempValue / 10;
                                            temperatures.Add(new PsychTemperature { timestamp = timestamp, value = value });
                                            healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 6, value = tempValue });
                                            sb.Append($"{tempTimestamp}|{tempValue}\r\n");
                                        }
                                        //步数
                                        sb.Append($"彼岸步数\r\n");
                                        offset += 28;
                                        int stepCnt = Utils.byteToInt(bt, offset, 4);//128
                                        for (int i = 0; i < stepCnt; i++)
                                        {
                                            uint stepTimestamp = Utils.byteToUint(bt, (offset + offsetLen + (i * 8)), 4);
                                            int stepValue = Utils.byteToInt(bt, (offset + offsetLen + 4 + (i * 8)), 4);
                                            int timestamp = (int)stepTimestamp;
                                            measures.Add(new PsychMeasure { timestamp = timestamp, type = 5, value = stepValue });
                                            healthData.Add(new PsychHealthDataInfo { index = i, timestamp = timestamp, type = 5, value = stepValue });
                                            sb.Append($"{stepTimestamp}|{stepValue}\r\n");
                                        }
                                        //ppg
                                        sb.Append($"彼岸ppg\r\n");
                                        offset += 52;
                                        int ppgCnt = Utils.byteToInt(bt, offset, 4);//180
                                        for (int i = 0; i < ppgCnt; i++)
                                        {
                                            List<int> vals = new List<int>();
                                            uint ppgTimestamp = 0;
                                            StringBuilder sbppg = new StringBuilder();
                                            for (int j = 0; j < 26; j++)
                                            {
                                                int set = offset + offsetLen * (j + 1) + (i * 104);
                                                if (j == 0)
                                                {
                                                    uint val = Utils.byteToUint(bt, set, 4);
                                                    ppgTimestamp = Utils.byteToUint(bt, set, 4);
                                                    sbppg.Append($"{val},");
                                                }
                                                else
                                                {
                                                    int val = Utils.byteToInt(bt, set, 4);
                                                    int new_val = val + 4096;
                                                    vals.Add(new_val);
                                                    sbppg.Append($"{new_val},");
                                                    //vals.Add(val);
                                                    //sbppg.Append($"{val},");
                                                }
                                            }
                                            int timestamp = (int)ppgTimestamp;
                                            int timestampPPG = (int)ppgTimestamp;
                                            ppg.Add(new PsychPPG { timestamp = timestamp, value = vals });
                                            bianPPG.Add(new BiAnPPG() { ppg = vals, time = timestampPPG, uid = $"{imei}" });
                                            sb.Append($"({i + 1}){sbppg.ToString()}\r\n");
                                        }
                                        //acc
                                        sb.Append($"彼岸acc\r\n");
                                        offset += 37444;
                                        int accCnt = Utils.byteToInt(bt, offset, 4);//180
                                        for (int i = 0; i < accCnt; i++)
                                        {
                                            List<double> vals = new List<double>();
                                            uint accTimestamp = 0;
                                            StringBuilder sbacc = new StringBuilder();
                                            for (int j = 0; j < 31; j++)
                                            {
                                                int set = offset + offsetLen * (j + 1) + (i * 124);
                                                if (j == 0)
                                                {
                                                    uint val = Utils.byteToUint(bt, set, 4);
                                                    accTimestamp = val;
                                                    sbacc.Append($"{val},");
                                                }
                                                else
                                                {
                                                    int val = Utils.byteToInt(bt, set, 4);
                                                    double avg = Math.Round((double)val / 1000, 4);
                                                    vals.Add(avg);
                                                    sbacc.Append($"{avg},");
                                                }
                                            }
                                            List<List<double>> value = Utils.SpiltList<double>(vals, 10);
                                            long timestamp = (long)accTimestamp * 1000;
                                            acc.Add(new PsychMotion { timestamp = timestamp, value = value });
                                            sb.Append($"({i + 1}){sbacc.ToString()}\r\n");
                                        }

                                        if (bt.Length == 75084)
                                        {
                                            sb.Append($"彼岸lbs\r\n"); //Lbs数据ascII码数据(256字符)
                                            offset += 37204;
                                            byte[] result = bt.Skip(offset).Take(256).ToArray();
                                            string lbsValue = Convert.ToString(Encoding.ASCII.GetString(result));//str为ASCII码对应的字符
                                            lbsValue = lbsValue.Replace("\0", ""); //去掉空格
                                            sb.Append($"{lbsValue}\r\n");
                                            lbs = lbsValue;
                                        }
                                        #endregion
                                    }
                                    //var dataInfo = new { timestamp = start_timestamp, rri = ppg };
                                    var dataInfo = new { timestamp = start_timestamp, ppg = bianPPG };
                                    var psycJson = new { psycBianRemarkObj.imei, psycBianRemarkObj.fileName, version = "", data = dataInfo };
                                    string psycJsonStr = JsonConvert.SerializeObject(psycJson);
                                    //数据采集解析保存
                                    string timeStamp = psycBianRemarkObj.fileName.Replace(".dat", "");
                                    try
                                    {
                                        var param = new GeneralParam
                                        {
                                            Filters = new List<QueryFilterCondition>{
                                      new QueryFilterCondition {
                                            Key=nameof(HisGpsPsych.Imei),
                                            Value=imei,
                                            ValueType=QueryValueTypeEnum.String,
                                            Operator=QueryOperatorEnum.Equal
                                         },
                                     new QueryFilterCondition {
                                            Key=nameof(HisGpsPsych.TimeStamp),
                                            Value=timeStamp,
                                            ValueType=QueryValueTypeEnum.Int,
                                            Operator=QueryOperatorEnum.Equal
                                         }
                                    }
                                        };
                                        var hisGpsPsych = _messagePsychClient.GetFirst(param);
                                        if (hisGpsPsych != null)
                                        {
                                            hisGpsPsych.FileDatData = sb.ToString();
                                            hisGpsPsych.FileDatDataTime = DateTime.Parse(model.time);
                                            await _messagePsychClient.UpdateAsync(hisGpsPsych);
                                        }
                                    }
                                    catch (Exception ex2)
                                    {
                                        _logger.LogError($"[彼岸心理数据解析结果保存<{obj.remark}>] 异常, {ex2.Message}|{ex2.Source}|{ex2.StackTrace}");
                                    }

                                    if (start_timestamp > 0)
                                    {
                                        await _psychHealthData.Save(imei, obj.msgId, healthData, "");//健康数据保存
                                        string psycReqquestUrl = $"{_abilityConfig.CoreServiceApiUrl}/OpenPsych/terminal/bian_fileuploaresult";
                                        var psycResponseStr = await _httpHelper.PostAsync(psycReqquestUrl, psycJsonStr);
                                        JObject psycJsons = JObject.FromObject(psycJson);
                                        psycJsons.Remove("data");

                                        _logger.LogInformation($"[彼岸心理数据文件(推送)第1次<{obj.remark}>]URL:{psycReqquestUrl}: 请求参数:{JsonConvert.SerializeObject(psycJsons)},返回结果:{psycResponseStr}");
                                        if (psycResponseStr.Contains("请求异常"))
                                        {
                                            psycResponseStr = await _httpHelper.PostAsync(psycReqquestUrl, psycJsonStr);//重新推送
                                            _logger.LogInformation($"[彼岸心理数据文件(推送异常)第2次<{obj.remark}>]URL:{psycReqquestUrl}: 请求参数:{JsonConvert.SerializeObject(psycJsons)},返回结果:{psycResponseStr}");
                                        }
                                    }
                                    else
                                        _logger.LogInformation($"[彼岸心理数据文件(解析)<{obj.remark}>]:{fileUrl}解析异常");
                                }
                                catch (Exception ex)
                                {
                                    _logger.LogError($"[彼岸心理数据异常<{obj.remark}>] 事件信息:{obj.remark}，异常,{ex.Message}|{ex.Source}|{ex.StackTrace}");
                                }
                            }
                            #endregion
                            break;
                        case "fence":
                            if (obj.status.ToLower() == "content_read" && !string.IsNullOrEmpty(obj.remark))
                            {
                                string businessId = obj.remark.Replace("更新成功!", "");
                                string iot_reply = JsonConvert.SerializeObject(model.data);
                                string iot_reply_time = model.time;
                                string iot_reply_push = "";

                                var settingInfo = await _redis.GetManufactorPushSettingHash(imei, manufactorId, 0);
                                if (settingInfo != null)
                                {
                                    Dictionary<string, object> dic = new Dictionary<string, object>();
                                    Dictionary<string, object> itemDic = new Dictionary<string, object>();
                                    if (_abilityConfig.RzlManufactorId == manufactorId)
                                    {
                                        dic.Add("imei", imei);
                                        dic.Add("alarmType", (int)MqAlarmType.DrownReportAlarm);
                                        itemDic.Add("imei", imei);
                                        itemDic.Add("msgId", model.messageId);
                                        itemDic.Add("msgType", "Fence");
                                        var reamrk = new { businessId, msg = "设备围栏信息更新成功", dateTime = TimeHelper.ConvertToTimeStamp(Convert.ToDateTime(model.time)) };
                                        itemDic.Add("remark", JsonConvert.SerializeObject(reamrk));
                                        itemDic.Add("timeStamp", TimeHelper.ConvertToTimeStamp(DateTime.Now));
                                        dic.Add("data", JsonConvert.SerializeObject(itemDic));
                                        dic.Add("dataTime", model.time);
                                        MultipartFormDataContent mfdc = Utils.GetMultipartFormDataContent(dic, _abilityConfig.RzlManufactorId, ref dic);
                                        var result = await _httpHelper.PostFormAsync(settingInfo.pushUrl, mfdc);
                                        _logger.LogInformation($"[设备围栏更新事件-RZL数据推送服务<{imei}>] url:{settingInfo.pushUrl},参数:{JsonConvert.SerializeObject(dic)},结果：{result}");

                                        iot_reply_push = $"[设备围栏更新事件-RZL数据推送服务<{imei}>] url:{settingInfo.pushUrl},参数:{JsonConvert.SerializeObject(dic)},结果：{result}";
                                    }
                                }
                                await _redis.UpdateAbilityEventDeviceBusiness(imei, businessId, iot_reply, iot_reply_time, iot_reply_push);

                            }
                            break;
                        case "blood_sugar":

                            break;
                    }
                }
                catch (Exception ex)
                {
                    _logger.LogError($"[设备学习能力状态<{imei}-{obj?.msgId}>] 异常,{ex.Message}|{ex.Source}|{ex.StackTrace}");
                }
            }
        }

        #endregion
    }
}
