﻿using Exceptionless.Models;
using Microsoft.Extensions.Options;
using Newtonsoft.Json;
using System.Text;
using TelpoDataService.Util.Clients;
using TelpoDataService.Util.Entities.GpsLocationHistory;
using TelpoDataService.Util.Models;
using TelpoDataService.Util.QueryObjects;
using TelpoPush.Ability.Worker.Common;
using TelpoPush.Ability.Worker.Models.CacheTemplates;
using TelpoPush.Ability.Worker.Models.Config;
using TelpoPush.Ability.Worker.Models.Entity.His;
using TelpoPush.Ability.Worker.Models.Enum;
using TelpoPush.Ability.Worker.Models.MqTemplates;
using TelpoPush.Ability.Worker.Models.MqTemplates.Psych;
using TelpoPush.Ability.Worker.Models.PushTemplates;
using TelpoPush.Ability.Worker.Service.Cache;
using TelpoPush.Ability.Worker.Service.Mq;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace TelpoPush.Ability.Worker.Service.Biz
{
    public class PsychHealthData
    {
        private readonly ILogger<PsychHealthData> _logger;
        private readonly AbilityConfig _abilityConfig;
        private readonly HttpHelperAsync _httpHelper;
        private readonly RedisUtil _redis;
        private readonly SqlMapper _sqlMapper;
        private readonly MqProcessMessage _serviceMqProcess;
        private readonly GpsLocationHistoryAccessorClient<HisGpsBloodPress> _messageBloodPressClient;
        private readonly GpsLocationHistoryAccessorClient<HisGpsHeartRate> _messageHeartRateClient;
        private readonly GpsLocationHistoryAccessorClient<HisGpsSpo2> _messageSpo2Client;
        private readonly GpsLocationHistoryAccessorClient<HisGpsTemperature> _messageTemperatureClient;
        private readonly GpsLocationHistoryAccessorClient<HisGpsStep> _messageStepClient;

        private readonly GpsLocationHistoryAccessorClient<HisGpsCiResult> _messageCiClient;
        private readonly GpsLocationHistoryAccessorClient<HisGpsMiResult> _messageMiClient;
        private readonly GpsLocationHistoryAccessorClient<HisGpsPsychData> _messagePsychDataClient;
        public PsychHealthData(ILogger<PsychHealthData> logger,
            IOptions<AbilityConfig> optAbilityConfig,
            HttpHelperAsync httpHelper,
            MqProcessMessage serviceMqProcess,
            RedisUtil redis,
            SqlMapper sqlMapper,
            GpsLocationHistoryAccessorClient<HisGpsBloodPress> messageBloodPressClient,
            GpsLocationHistoryAccessorClient<HisGpsHeartRate> messageHeartRateClient,
            GpsLocationHistoryAccessorClient<HisGpsSpo2> messageSpo2Client,
            GpsLocationHistoryAccessorClient<HisGpsTemperature> messageTemperatureClient,
            GpsLocationHistoryAccessorClient<HisGpsStep> messageStepClient,
            GpsLocationHistoryAccessorClient<HisGpsCiResult> messageCiClient,
            GpsLocationHistoryAccessorClient<HisGpsMiResult> messageMiClient,
            GpsLocationHistoryAccessorClient<HisGpsPsychData> messagePsychDataClient
            )
        {
            _logger = logger;
            _abilityConfig = optAbilityConfig.Value;
            _httpHelper = httpHelper;
            _serviceMqProcess = serviceMqProcess;
            _redis = redis;
            _sqlMapper = sqlMapper;
            _messageBloodPressClient = messageBloodPressClient;
            _messageHeartRateClient = messageHeartRateClient;
            _messageSpo2Client = messageSpo2Client;
            _messageTemperatureClient = messageTemperatureClient;
            _messageStepClient = messageStepClient;
            _messageMiClient= messageMiClient;
            _messageCiClient = messageCiClient;
            _messagePsychDataClient = messagePsychDataClient;
        }

        public async Task Save(string imei, string messageId, List<PsychHealthDataInfo> healthDatas, string ppg)
        {
            string deviceKey = await _redis.GetHealthyDeviceKey(imei);

            //12 血压, 3 血氧, 4 心率, 5 步数, 6 体温
            //体温低于于34.5度，心率低于40都不显示不推送。分析结果失败问题还需继续跟进  20231225 需求
            //今天优先处理一下健康数据的体温、心率地低的值的不显示，其他都显示并推送给第三方
            int heartRateVal = 40;
            int temperatureVal = 340;
            bool isUsable = false;//全部不显示、不推送，

            /*    
             *     20240408 注释
             *     
             *    bool isUsable = true; 
                        int step = 0;
                        var steps = await _redisBaseManager.GetStepsHash(imei);
                        if (steps != null)
                        {
                            step = steps.Steps;
                            if (!string.IsNullOrEmpty(steps.LastUpdate))
                            {
                                double hours = (DateTime.Now - DateTime.Parse(steps.LastUpdate)).TotalHours;
                                if (hours > 1)
                                    isUsable = false;
                            }
                        }
                        if (isUsable && step <= 0)
                            isUsable = false;*/
            string subImei = imei.Substring(imei.Length - 3);//截取IMEI最后３个字符

            #region 心脑监测业务 1 
            CiMiValueModel hisDate = new CiMiValueModel();
            hisDate.imei = imei;
            hisDate.date = TimeHelper.ToDateStr(DateTime.Now);
            hisDate.Count = 1;
            hisDate.last_update = TimeHelper.ToDateTimeStr(DateTime.Now);
            StringBuilder sb = new StringBuilder();
            StringBuilder info = new StringBuilder();
            bool isCiMi = true; 
            int miVal = 0, ciVal = 0, stepTotal = 0;
            bool isBloodPress = false, isSpo2 = false;
            var person = await _sqlMapper.GetHealthyPerson(imei);
            if (person != null)
            {
                //person.medical_history = "1,2,3,4,5,6,7,8,9";
                //person.lifestyle = "1";
                string personInfo = "";
                if (person.birth.HasValue)
                {
                    //65~74	1	1
                    //>= 75 2   2
                    bool isAge = false;
                    int age = DateTime.Now.Year - person.birth.Value.Year;
                    if (age >= 65 && age <= 74)
                    {
                        hisDate.birth = [1, 1];
                        sb.Append($"birth({TimeHelper.ToDateStr(person.birth.Value)},age={age})[mi=1,ci=1],\r\n"); miVal += 1; ciVal += 1; isAge = true;
                    }
                    else if (age >= 75)
                    {
                        hisDate.birth = [2, 2];
                        sb.Append($"birth({TimeHelper.ToDateStr(person.birth.Value)},age={age})[mi=2,ci=2],\r\n"); miVal += 2; ciVal += 2; isAge = true;
                    }
                    else
                    {
                        hisDate.birth = [0, 0];
                        sb.Append($"birth({TimeHelper.ToDateStr(person.birth.Value)},age={age})[mi=0,ci=0],\r\n");
                    }
                    if (person.gender == 2 && isAge)
                    {
                        hisDate.gender = [1, 1];
                        sb.Append($"gender({person.gender},1=男,2=女)[mi=1,ci=1],\r\n"); miVal += 1; ciVal += 1;
                    }
                    else
                    {
                        hisDate.gender = [0, 0];
                        sb.Append($"gender({person.gender},1=男,2=女)[mi=0,ci=0],\r\n");
                    }
                    personInfo += $"{age}|{person.gender}|";
                }
                else
                {
                    sb.Append($"birth(未设置),gender({person.gender},1=男,2=女)[mi=0,ci=0],\r\n");
                    personInfo += $"{0}|{person.gender}|";
                }
                if (!string.IsNullOrEmpty(person.medical_history))
                {
                    //病史		
                    //心梗	3	2
                    //糖尿病	3	3
                    //高脂血症	3	0  ----血症
                    //冠心病	3	3
                    //心衰	2	2
                    //腔隙性脑梗	2	2
                    //高尿酸血症	2	2  ---- 高尿酸
                    //脑卒中	0	3

                    string[] vals = person.medical_history.Split(',');
                    if (vals.Length > 0)
                    {
                        var medicalHistory = new Dictionary<int, string>() {
                         {0,"无病史"},{1,"心梗"},{2,"脑卒中"},{3,"腔隙性脑梗"},
                         {4,"高血糖"},{5,"冠心病"}, {6,"高血压"}, {7,"糖尿病"},
                         {8,"高尿酸"},{9,"血症"},{10,"心衰"},{11,"肿瘤"},
                         {12,"阿尔兹海默症"},{13,"帕金森氏病"},{14,"慢阻肺"}, {15,"哮喘"},
                         {16,"尘肺病"}, {17,"胃炎"},   {18,"胃溃疡"},  {19,"胃癌"}};

                        foreach (var item in vals)
                        {
                            int val = int.Parse(item);
                            if (val == 1 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_1 = [3, 2];
                                sb.Append($"心梗({val})[mi=3,ci=2],\r\n"); miVal += 3; ciVal += 2;
                            }
                            if (val == 2 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_2 = [0, 3];
                                sb.Append($"脑卒中({val})[mi=0,ci=3],\r\n"); miVal += 0; ciVal += 3;
                            }
                            if (val == 3 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_3 = [2, 2];
                                sb.Append($"腔隙性脑梗({val})[mi=2,ci=2],\r\n"); miVal += 2; ciVal += 2;
                            }
                            if (val == 5 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_5 = [3, 3];
                                sb.Append($"冠心病({val})[mi=3,ci=3],\r\n"); miVal += 3; ciVal += 3;
                            }
                            if (val == 7 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_7 = [3, 3];
                                sb.Append($"糖尿病({val})[mi=3,ci=3],\r\n"); miVal += 3; ciVal += 3;
                            }
                            if (val == 8 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_8 = [2, 2];
                                sb.Append($"高尿酸({val})[mi=2,ci=2],\r\n"); miVal += 2; ciVal += 2;
                            }
                            if (val == 9 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_9 = [3, 0];
                                sb.Append($"血症({val})[mi=3,ci=0],\r\n"); miVal += 3; ciVal += 0;
                            }
                            if (val == 10 && medicalHistory.ContainsKey(val))
                            {
                                hisDate.mh_10 = [2, 2];
                                sb.Append($"心衰({val})[mi=2,ci=2],\r\n"); miVal += 2; ciVal += 2;
                            }
                            if (val == 0)
                                sb.Append($"无病史({val})[mi=0,ci=0],\r\n");
                        }
                    }
                }
                else
                    sb.Append($"病史(未设置)[mi=0,ci=0],\r\n");

                if (!string.IsNullOrEmpty(person.lifestyle))
                {
                    //生活方式		
                    //吸烟	1	1
                    //饮酒	1	1
                    string[] vals = person.lifestyle.Split(',');
                    if (vals.Length > 0)
                    {
                        var lifestyle = new Dictionary<int, string>() {
                            {0,"无不良习惯"},{1,"吸烟"},{2,"饮酒"}};
                        foreach (var item in vals)
                        {
                            int val = int.Parse(item);
                            if (val == 1 && lifestyle.ContainsKey(val))
                            {
                                hisDate.ls_1 = [1, 1];
                                sb.Append($"吸烟({val})[mi=1,ci=1],\r\n"); miVal += 1; ciVal += 1;
                            }
                            if (val == 2 && lifestyle.ContainsKey(val))
                            {
                                hisDate.ls_2 = [1, 1];
                                sb.Append($"饮酒({val})[mi=1,ci=1],\r\n"); miVal += 1; ciVal += 1;
                            }
                            if (val == 0)
                                sb.Append($"无不良习惯({val})[mi=0,ci=0],\r\n");
                        }
                    }
                }
                else
                    sb.Append($"生活方式(未设置)[mi=0,ci=0],\r\n");
                personInfo += $"{person.medical_history?.Trim(',')}|{person.lifestyle?.Trim(',')}";
                hisDate.personInfo = personInfo;
                if (string.IsNullOrEmpty(person.medical_history) && string.IsNullOrEmpty(person.lifestyle)) {
                    isCiMi = false; info.Append($"telpo_healthy,person病史信息,生活方式信息为空(心脑未开通),");
                }
            }
            else
            {
                isCiMi = false; info.Append($"设备解绑或telpo_healthy,person信息为空,");
                sb.Append($"设备{imei} telpo_healthy,person信息为空,");
            }
            string bloodPressVals = "", systolicVals = "", spo2Vals = "", heartRateVals = "", stepVals = "", temperatureVals = "";

            #endregion
            foreach (var item in healthDatas)
            {
                DateTime lastUpdate = TimeHelper.ConvertTimeStampToDateTime(item.timestamp.ToString());
                if (item.type == 12)
                {
                    #region  心脑监测值计算(血压)
                    //141~160	1	1
                    // 161~180 2   2
                    // >= 181   3   3
                    bloodPressVals += $"{item.value}|{item.value2},";
                    systolicVals += $"{item.value},";
                    bool isData = false;
                    if (item.value >= 181)
                    {
                        hisDate.systolic = [3, 3];
                        sb.Append($"systolic({systolicVals})[mi=3,ci=3],\r\n"); miVal += 3; ciVal += 3; isData = true; isBloodPress = true;
                    }
                    if (!isData)
                    {
                        if (item.value >= 161 && item.value <= 180)
                        {
                            hisDate.systolic = [2, 2];
                            sb.Append($"systolic({systolicVals})[mi=2,ci=2],\r\n"); miVal += 2; ciVal += 2; isData = true; isBloodPress = true;
                        }
                    }
                    if (!isData)
                    {
                        if (item.value >= 141 && item.value <= 160)
                        {
                            hisDate.systolic = [1, 1];
                            sb.Append($"systolic({systolicVals})[mi=1,ci=1],\r\n"); miVal += 1; ciVal += 1; isBloodPress = true;
                        }
                    }
                    #endregion

                    HisGpsBloodPress bloodPress = new HisGpsBloodPress()
                    {
                        DeviceKey = deviceKey,
                        BloodPressId = $"{string.Format("{0:yyMMddHHmmssffff}", DateTime.Now)}_{subImei}_psych",
                        MessageId = messageId,
                        Serialno = imei,
                        SystolicValue = item.value, //收缩压
                        DiastolicValue = item.value2,//舒张压
                        Method = 2,
                        IsDisplay = isUsable ? 1 : 0,
                        LastUpdate = lastUpdate,
                        CreateTime = DateTime.Now,
                    };
                    string requestId = $"psych_bloodPress_{messageId}_{item.index}";
                    await _messageBloodPressClient.AddAsync(bloodPress, imei, new RequestHeader() { RequestId = requestId });

                    #region kafka推送
                    if (isUsable)
                    {
                        Dictionary<string, int> dicHeader = new Dictionary<string, int>();
                        dicHeader.Add(MqHeader.DataType, (int)MqDataType.BloodPress1Info);
                        BaseModel pushData = new BaseModel()
                        {
                            messageId = bloodPress.BloodPressId,
                            topic = "topic.third",
                            time = TimeHelper.ToDateTimeStr(lastUpdate),
                            data = new PushBloodPressInfo()
                            {
                                imei = imei,
                                systolicValue = item.value,
                                diastolicValue = item.value2
                            }
                        };
                        await _serviceMqProcess.ProcessThirdhServer(imei, pushData, dicHeader, $"心理监测-血压({item.index})");
                        Dictionary<string, string> header = new Dictionary<string, string>();
                        header.Add("RequestId", requestId);
                        string jsonStr = JsonConvert.SerializeObject(new
                        {
                            BloodPressId = bloodPress.BloodPressId,
                            MessageId = messageId,
                            Serialno = imei,
                            SystolicValue = item.value,
                            DiastolicValue = item.value2,
                            Method = 2,
                            IsDisplay = 0,
                            LastUpdate = TimeHelper.ToDateTimeStr(lastUpdate),
                            CreateTime = TimeHelper.ToDateTimeStr(bloodPress.CreateTime.Value),
                        });
                        var result = await _httpHelper.PostAsync(_abilityConfig.HmBloodPressAddUrl, jsonStr, header);
                        _logger.LogInformation($"心理监测-血压HmBloodPressAdd保存<{imei}>] Url:{_abilityConfig.HmBloodPressAddUrl};参数：{jsonStr};结果:{result}");
                    }
                    #endregion
                }
                if (item.type == 3)
                {
                    #region  心脑监测值计算(血氧)
                    // 94%	1	1
                    // 93 % 2   2
                    // <= 92 % 3   3
                    spo2Vals += $"{item.value},";
                    bool isData = false;
                    if (item.value <= 92)
                    {
                        hisDate.spo2 = [3, 3];
                        sb.Append($"spo2({spo2Vals})[mi=3,ci=3],\r\n"); miVal += 3; ciVal += 3; isData = true; isSpo2 = true;
                    }
                    if (!isData)
                    {
                        if (item.value == 93)
                        {
                            hisDate.spo2 = [2, 2];
                            sb.Append($"spo2({spo2Vals})[mi=2,ci=2],\r\n"); miVal += 2; ciVal += 2; isData = true; isSpo2 = true;
                        }
                    }
                    if (!isData)
                    {
                        if (item.value == 94)
                        {
                            hisDate.spo2 = [1, 1];
                            sb.Append($"spo2({spo2Vals})[mi=1,ci=1],\r\n"); miVal += 1; ciVal += 1; isSpo2 = true;
                        }
                    }
                    #endregion

                    HisGpsSpo2 spo2 = new HisGpsSpo2()
                    {
                        DeviceKey = deviceKey,
                        Spo2Id = $"{string.Format("{0:yyMMddHHmmssffff}", DateTime.Now)}_{subImei}_psych",
                        MessageId = messageId,
                        Serialno = imei,
                        Spo2 = item.value,
                        Method = 2,
                        IsDisplay = isUsable ? 1 : 0,
                        LastUpdate = lastUpdate,
                        CreateTime = DateTime.Now,
                    };
                    string requestId = $"psych_spo2_{messageId}_{item.index}";
                    await _messageSpo2Client.AddAsync(spo2, imei, new RequestHeader() { RequestId = requestId });

                    #region kafka推送
                    if (isUsable)
                    {
                        Dictionary<string, int> dicHeader = new Dictionary<string, int>();
                        dicHeader.Add(MqHeader.DataType, (int)MqDataType.Spo21Info);
                        BaseModel pushData = new BaseModel()
                        {
                            messageId = spo2.Spo2Id,
                            topic = "topic.third",
                            time = TimeHelper.ToDateTimeStr(lastUpdate),
                            data = new PushDeviceSpo2Info()
                            {
                                imei = imei,
                                spo2 = item.value
                            }
                        };
                        await _serviceMqProcess.ProcessThirdhServer(imei, pushData, dicHeader, $"心理监测-血氧({item.index})");
                    }
                    #endregion
                }
                if (item.type == 4)
                {
                    heartRateVals += $"{item.value},";
                    bool isPush = false;
                    if (item.value > heartRateVal && isUsable)
                        isPush = true;
                    HisGpsHeartRate heartRate = new HisGpsHeartRate()
                    {
                        DeviceKey = deviceKey,
                        HeartRateId = $"{string.Format("{0:yyMMddHHmmssffff}", DateTime.Now)}_{subImei}_psych",
                        MessageId = messageId,
                        Serialno = imei,
                        HeartRate = item.value,
                        Method = 2,
                        IsDisplay = isUsable ? 1 : 0,
                        LastUpdate = lastUpdate,
                        CreateTime = DateTime.Now,
                    };
                    string requestId = $"psych_heartRate_{messageId}_{item.index}";
                    await _messageHeartRateClient.AddAsync(heartRate, imei, new RequestHeader() { RequestId = requestId });
                    #region kafka推送
                    if (isPush)
                    {
                        Dictionary<string, int> dicHeader = new Dictionary<string, int>();
                        dicHeader.Add(MqHeader.DataType, (int)MqDataType.HeartRate1Info);
                        BaseModel pushData = new BaseModel()
                        {
                            messageId = heartRate.HeartRateId,
                            topic = "topic.push.third",
                            time = TimeHelper.ToDateTimeStr(lastUpdate),
                            data = new PushHeartRateInfo()
                            {
                                imei = imei,
                                heartRate = item.value
                            }
                        };
                        await _serviceMqProcess.ProcessThirdhServer(imei, pushData, dicHeader, $"心理监测-心率({item.index})");
                    }
                    #endregion
                }
                if (item.type == 5)
                {
                    stepTotal += item.value;
                    stepVals += $"{item.value},";

                    /*                    if (item.value > 0)
                                        {
                                            HisGpsStep step = new HisGpsStep()
                                            {
                                                DeviceKey = deviceKey,
                                                StepId = $"{string.Format("{0:yyMMddHHmmssffff}", DateTime.Now)}_{messageId}_{subImei}_psych",
                                                Serialno = imei,
                                                Steps = item.value,
                                                LastUpdate = lastUpdate,
                                            };
                                            string requestId = $"psych_step_{messageId}_{item.index}";
                                            await _messageStepClient.AddAsync(step, imei, new RequestHeader() { RequestId = requestId });
                                        }*/
                }
                if (item.type == 6)
                {
                    temperatureVals += $"{item.value},";

                    bool isPush = false;
                    if (item.value > temperatureVal && isUsable)
                        isPush = true;
                    HisGpsTemperature temperature = new HisGpsTemperature()
                    {
                        DeviceKey = deviceKey,
                        TemperatureId = $"{string.Format("{0:yyMMddHHmmssffff}", DateTime.Now)}_{subImei}_psych",
                        TempId = messageId,
                        Serialno = imei,
                        Temperature = (decimal)item.value / 10,
                        Method = 2,
                        IsDisplay = isUsable ? 1 : 0,
                        LastUpdate = lastUpdate
                    };
                    string requestId = $"psych_temperature_{messageId}_{item.index}";
                    await _messageTemperatureClient.AddAsync(temperature, imei, new RequestHeader() { RequestId = requestId });
                    #region kafka推送
                    if (isPush)
                    {
                        Dictionary<string, int> dicHeader = new Dictionary<string, int>();
                        dicHeader.Add(MqHeader.DataType, (int)MqDataType.Temperature1Info);
                        BaseModel pushData = new BaseModel()
                        {
                            messageId = temperature.TemperatureId,
                            topic = "topic.push.third",
                            time = TimeHelper.ToDateTimeStr(lastUpdate),
                            data = new PushTemperatureInfo()
                            {
                                Imei = imei,
                                TempId = messageId,
                                Temperature = temperature.Temperature
                            }
                        };
                        await _serviceMqProcess.ProcessThirdhServer(imei, pushData, dicHeader, $"心理监测-体温({item.index})");
                    }
                    #endregion
                }
            }

            #region 心脑监测业务 2 
            if (!isBloodPress)
            {
                hisDate.systolic = [0, 0];
                sb.Append($"systolic({systolicVals})[mi=0,ci=0],\r\n");
            }
            if (!isSpo2)
            {
                hisDate.spo2 = [0, 0];
                sb.Append($"spo2({spo2Vals})[mi=0,ci=0],\r\n");
            }
            if (stepTotal == 0)
            {
                isCiMi = false; info.Append($"步数总数为0,本次不做统计,");
            }
            var heartRateObj = healthDatas.Where(s => s.type == 4 && s.value >= 160).ToList();
            if (heartRateObj?.Count > 0 && stepTotal <= 300)
            {
                hisDate.heartRate = [1, 1];
                sb.Append($"heartRate({heartRateVals})Step({stepVals}total={stepTotal})[mi=1,ci=1],\r\n"); miVal += 1; ciVal += 1;
            }
            else
            {
                hisDate.heartRate = [0, 0];
                sb.Append($"heartRate({heartRateVals})Step({stepVals}total={stepTotal}))[mi=0,ci=0],\r\n");
            }
            hisDate.total = [miVal, ciVal];

            DateTime dt = DateTime.Now;
            try
            {
                sb.Append($"合计:[mi={miVal},ci={ciVal}]");

                HisGpsPsychData psychData = new HisGpsPsychData()
                {
                    PsychId = messageId,
                    PsychType = 1,
                    SerialNo = imei,
                    BloodPress = bloodPressVals.Trim(','),
                    Spo2 = spo2Vals.Trim(','),
                    HeartRate = heartRateVals.Trim(','),
                    Temperature = temperatureVals.Trim(','),
                    Step = $"{stepVals.Trim(',')}|{stepTotal}",
                    Remark = sb.ToString(),
                    IndexNum = 1,
                    Info = JsonConvert.SerializeObject(hisDate),
                    LastUpdate = dt,
                    CreateTime = dt,
                    Ppg = ppg,
                };

                if (!isCiMi)
                {
                    psychData.IndexNum = 0;
                    psychData.Info = info.ToString().Trim(',');
                    await _messagePsychDataClient.AddAsync(psychData);
                }
                else
                {
                    var cimi = await CiMiValue(hisDate);
                    if (cimi != null)
                    {
                        psychData.IndexNum = cimi.Count;
                        psychData.Info = JsonConvert.SerializeObject(cimi);
                    }
                    await _messagePsychDataClient.AddAsync(psychData);
                    if (cimi?.Count >= 4)//当天第4次采集数据后进行心脑统计
                    {
                   
                        //mi
                        var miParam = new GeneralParam
                        {
                            Filters = new List<QueryFilterCondition>{
                                      new QueryFilterCondition {
                                            Key=nameof(HisGpsMiResult.SerialNo),
                                            Value=imei,
                                            ValueType=QueryValueTypeEnum.String,
                                            Operator=QueryOperatorEnum.Equal
                                         },
                                     new QueryFilterCondition {
                                            Key=nameof(HisGpsMiResult.LastUpdate),
                                            Value =TimeHelper.ToDay00_00(dt).ToString(),
                                            ValueType=QueryValueTypeEnum.DateTime,
                                            Operator=QueryOperatorEnum.GreaterEqual
                                         },
                                     new QueryFilterCondition {
                                            Key=nameof(HisGpsMiResult.LastUpdate),
                                            Value=TimeHelper.ToDay23_59(dt).ToString(),
                                            ValueType=QueryValueTypeEnum.DateTime,
                                            Operator=QueryOperatorEnum.LessEqual
                                         }
                                    }
                        };
                        var mi = _messageMiClient.GetFirst(miParam);
                        if (mi != null)
                            _messageMiClient.Delete(mi);
                        HisGpsMiResult miresult = new HisGpsMiResult()
                        {
                            MiId = Guid.NewGuid().ToString(),
                            SerialNo = imei,
                            MiValue = cimi.total[0],
                            DeviceKey = deviceKey,
                            Remark = messageId,
                            LastUpdate = dt,
                            Method = 1,
                            Leve = 0,
                            IsDisplay = 1,
                        };
                        _messageMiClient.Add(miresult);

                        //ci
                        var ciParam = new GeneralParam
                        {
                            Filters = new List<QueryFilterCondition>{
                                      new QueryFilterCondition {
                                            Key=nameof(HisGpsCiResult.SerialNo),
                                            Value=imei,
                                            ValueType=QueryValueTypeEnum.String,
                                            Operator=QueryOperatorEnum.Equal
                                         },
                                     new QueryFilterCondition {
                                            Key=nameof(HisGpsCiResult.LastUpdate),
                                            Value =TimeHelper.ToDay00_00(dt).ToString(),
                                            ValueType=QueryValueTypeEnum.DateTime,
                                            Operator=QueryOperatorEnum.GreaterEqual
                                         },
                                     new QueryFilterCondition {
                                            Key=nameof(HisGpsCiResult.LastUpdate),
                                            Value=TimeHelper.ToDay23_59(dt).ToString(),
                                            ValueType=QueryValueTypeEnum.DateTime,
                                            Operator=QueryOperatorEnum.LessEqual
                                         }
                                    }
                        };
                        var ci = _messageCiClient.GetFirst(ciParam);
                        if (ci != null)
                            _messageCiClient.Delete(ci);
                        HisGpsCiResult ciResult = new HisGpsCiResult()
                        {
                            CiId = Guid.NewGuid().ToString(),
                            SerialNo = imei,
                            CiValue = cimi.total[1],
                            DeviceKey = deviceKey,
                            Remark = messageId,
                            LastUpdate = dt,
                            Method = 1,
                            Leve = 0,
                            IsDisplay = 1,
                        };
                        _messageCiClient.Add(ciResult);
                    }
                }
            }
            catch (Exception ex)
            {
                _logger.LogInformation($"心脑数据处理异常-<{imei}>] 说明：{ex.Message}");
            }
            #endregion
        }

        //心脑数据统计（缓存）
        private async Task<CiMiValueModel> CiMiValue(CiMiValueModel model)
        {
            var cimi = await _redis.GetGpsDeviceCiMiData(model.imei);
            if (cimi != null)
            {
                if (cimi.date != model.date)
                {
                    await _redis.SetGpsDeviceCiMiDataHash(model.imei, model);
                    return model;
                }
                cimi.imei = model.imei;
                cimi.personInfo = model.personInfo;
                cimi.date = model.date;
                cimi.Count = cimi.Count + model.Count;
                cimi.last_update = model.last_update;
                for (int i = 0; i < 2; i++)
                {
                    cimi.birth[i] = model.birth[i] > cimi.birth[i] ? model.birth[i] : cimi.birth[i];
                    cimi.gender[i] = model.gender[i] > cimi.gender[i] ? model.gender[i] : cimi.gender[i];
                    cimi.mh_1[i] = model.mh_1[i] > cimi.mh_1[i] ? model.mh_1[i] : cimi.mh_1[i];
                    cimi.mh_2[i] = model.mh_2[i] > cimi.mh_2[i] ? model.mh_2[i] : cimi.mh_2[i];
                    cimi.mh_3[i] = model.mh_3[i] > cimi.mh_3[i] ? model.mh_3[i] : cimi.mh_3[i];
                    cimi.mh_5[i] = model.mh_5[i] > cimi.mh_5[i] ? model.mh_5[i] : cimi.mh_5[i];
                    cimi.mh_7[i] = model.mh_7[i] > cimi.mh_7[i] ? model.mh_7[i] : cimi.mh_7[i];
                    cimi.mh_8[i] = model.mh_8[i] > cimi.mh_8[i] ? model.mh_8[i] : cimi.mh_8[i];
                    cimi.mh_9[i] = model.mh_9[i] > cimi.mh_9[i] ? model.mh_9[i] : cimi.mh_9[i];
                    cimi.mh_10[i] = model.mh_10[i] > cimi.mh_10[i] ? model.mh_10[i] : cimi.mh_10[i];
                    cimi.ls_1[i] = model.ls_1[i] > cimi.ls_1[i] ? model.ls_1[i] : cimi.ls_1[i];
                    cimi.ls_2[i] = model.ls_2[i] > cimi.ls_2[i] ? model.ls_2[i] : cimi.ls_2[i];
                    cimi.spo2[i] = model.spo2[i] > cimi.spo2[i] ? model.spo2[i] : cimi.spo2[i];
                    cimi.systolic[i] = model.systolic[i] > cimi.systolic[i] ? model.systolic[i] : cimi.systolic[i];
                    cimi.heartRate[i] = model.heartRate[i] > cimi.heartRate[i] ? model.heartRate[i] : cimi.heartRate[i];
                    cimi.total[i] = new[] { cimi.birth, cimi.gender, cimi.mh_1, cimi.mh_2, cimi.mh_3, cimi.mh_5, cimi.mh_7, cimi.mh_8, cimi.mh_9, cimi.mh_10, cimi.ls_1, cimi.ls_2, cimi.spo2, cimi.systolic, cimi.heartRate }.Select(arr => arr[i]).Sum();
                }
                await _redis.SetGpsDeviceCiMiDataHash(model.imei, cimi);
                return cimi;
            }
            await _redis.SetGpsDeviceCiMiDataHash(model.imei, model);
            return model;
        }

        public async Task Push(string imei, MqDataType dataType, BaseModel pushData)
        {
            Dictionary<string, int> dicHeader = new Dictionary<string, int>();
            dicHeader.Add(MqHeader.DataType, (int)dataType);
            await _serviceMqProcess.ProcessThirdhServer(imei, pushData, dicHeader, $"心理监测事件重试)");
        }
    }
}
