﻿using Dapper;
using MySql.Data.MySqlClient;
using System.Data;
using TelpoPush.Ability.Worker.Common;
using TelpoPush.Ability.Worker.Models.CacheTemplates;
using TelpoPush.Ability.Worker.Models.Entity;
using TelpoPush.Ability.Worker.Models.Entity.His;
using static System.Runtime.InteropServices.JavaScript.JSType;

namespace TelpoPush.Ability.Worker.Service.Cache
{
    public class SqlMapper
    {
        private readonly IConfiguration _config;
        private static string gps_conn = "";
        private static string telcommon_conn = "";
        private static string healthy_conn = "";
        private static string his_conn = "";
        public SqlMapper(IConfiguration config)
        {
            _config = config;
            gps_conn = _config["ConnectionStrings:DB_Connection_String"].ToString();
            telcommon_conn = _config["ConnectionStrings:Telpo_common_ConnString"].ToString();
            healthy_conn = _config["ConnectionStrings:Telpo_Healthy_ConnString"].ToString();
            string dateTime = DateTime.Now.ToString("yyyyMM");
            string dbName = $"location_{dateTime}";
            his_conn = _config["ConnectionStrings:Telpo_His_ConnString"].ToString().Replace("{dbName}", dbName, StringComparison.OrdinalIgnoreCase);
        }
        public async Task<int> GetPsychImei(string imei)
        {
            using (IDbConnection connection = new MySqlConnection(gps_conn))
            {
                var sql = @"SELECT COUNT(*) FROM gps_card.gps_psych_imei WHERE psych_type_id=2 AND `status`=1 AND imei=@imei";
                int icount = await connection.ExecuteScalarAsync<int>(sql, new { imei });
                return icount;
            }
        }


        public DeviceInfoModel DeviceInfo(string imei)
        {
            using (IDbConnection connection = new MySqlConnection(gps_conn))
            {
                var sql = @"SELECT d.device_id deviceId, p.nick_name deviceName,p.api_uid apiUid,d.serialno imei, d.org_uid orgId, d.active_status activeStatus,active_time activeTime FROM gps_device as d LEFT JOIN `gps_person` as p on p.device_id=d.device_id WHERE d.serialno=@imei";
                return connection.QueryFirstOrDefault<DeviceInfoModel>(sql, new { imei });
            }
        }
        public PersonModel PersonInfo(string imei)
        {
            using (IDbConnection connection = new MySqlConnection(gps_conn))
            {
                var sql = @"SELECT person_id personId, serialno, person_name personName, device_id deviceId, nick_name nickName, gender,  height, weight, born_date bornDate, school, grade, class_name className, image_path imagePath,image_path_small imagePathSmall, age, create_time createTime,  remarks, ishypertension, emotion,profession,regularity,chronic_disease chronicDisease,api_uid apiUid,api_remark apiRemark FROM gps_person WHERE serialno=@imei";
                return connection.QueryFirstOrDefault<PersonModel>(sql, new { imei });
            }
        }

        public async Task<int> UpdateAbilityEventDeviceBusiness(string imei, string businessid, string iot_reply, string iot_reply_time, string iot_reply_push)
        {
            using (IDbConnection connection = new MySqlConnection(gps_conn))
            {
                string sql = @"update gsp_ability_event_device_business set iot_reply_status=1, iot_reply=@iot_reply, iot_reply_time=@iot_reply_time, iot_reply_push=@iot_reply_push where imei=@imei and business_id=@businessid;";
                return await connection.ExecuteAsync(sql, new { imei, businessid, iot_reply, iot_reply_time, iot_reply_push });
            }
        }

        public ManufactorImeiModel GetManufactorImei(string imei)
        {
            using (IDbConnection connection = new MySqlConnection(telcommon_conn))
            {
                var sql = @"SELECT im.manufactor_id AS manufactorId ,im.manufactor_name AS manufactorName,im.imei, infos.app_id AS appId,infos.app_secret AS appSecret,infos.open_id AS openId FROM manufactor_imei  AS  im LEFT JOIN manufactor_info AS infos ON im.manufactor_id=infos.manufactor_id WHERE imei=@imei";
                return connection.QueryFirstOrDefault<ManufactorImeiModel>(sql, new { imei });
            }
        }

        public async Task<int> AddHisPsychData(GpsPsychDatas data)
        {
            using (IDbConnection connection = new MySqlConnection(his_conn))
            {
                string sql = @"insert into gps_psych_data(  psych_id, psych_type, serialno, blood_press, spo2, heart_rate, temperature, step, ppg, rri, remark, index_num, info, last_update, create_time) values ( @psych_id, @psych_type, @serialno, @blood_press, @spo2, @heart_rate, @temperature, @step, @ppg, @rri, @remark, @index_num, @info, @last_update, @create_time  );";
                return await connection.ExecuteAsync(sql, data);
            }
        }

        public async Task<int> AddHisCiResult(GpsCiResults data)
        {
            DateTime dt = DateTime.Now;
            data.ci_id = Guid.NewGuid().ToString();
            data.last_update = dt;
            data.method = 1;
            data.leve = 0;
            data.is_display = 1;
            using (IDbConnection connection = new MySqlConnection(his_conn))
            {
                string date = TimeHelper.ToDateStr(dt);
                var sql = @"select count(*) from gps_ci_result where serialno=@serialno and DATE_FORMAT(last_update, '%Y-%m-%d')=@date";
                string delete = @"delete from gps_ci_result where serialno=@serialno and DATE_FORMAT(last_update, '%Y-%m-%d')=@date;";
                string insert = @"insert into gps_ci_result(ci_id, serialno, ci_value, leve, remark, last_update, method, is_display, device_key) values  (@ci_id, @serialno, @ci_value, @leve, @remark, @last_update, @method, @is_display, @device_key );";
                int count = await connection.QueryFirstOrDefaultAsync<int>(sql, new { data.serialno, date });
                if (count > 0)
                    await connection.ExecuteAsync(delete, new { data.serialno, date });
                return await connection.ExecuteAsync(insert, data);
            }
        }

        public async Task<int> AddHisMiResult(GpsMiResults data)
        {
            DateTime dt = DateTime.Now;
            data.mi_id = Guid.NewGuid().ToString();
            data.last_update = dt;
            data.method = 1;
            data.leve = 0;
            data.is_display = 1;
            using (IDbConnection connection = new MySqlConnection(his_conn))
            {
                string date = TimeHelper.ToDateStr(dt);
                var sql = @"select count(*) from gps_mi_result where serialno=@serialno and DATE_FORMAT(last_update, '%Y-%m-%d')=@date";
                string delete = @"delete from  gps_mi_result where serialno=@serialno and DATE_FORMAT(last_update, '%Y-%m-%d')=@date;";
                string insert = @"insert into  gps_mi_result(mi_id, serialno, mi_value, leve, remark, last_update, method, is_display, device_key) values (@mi_id, @serialno, @mi_value, @leve, @remark, @last_update, @method, @is_display,@device_key );";
                int count = await connection.QueryFirstOrDefaultAsync<int>(sql, new { data.serialno, date });
                if (count > 0)
                    await connection.ExecuteAsync(delete, new { data.serialno, date });
                return await connection.ExecuteAsync(insert, data);
            }
        }


        public async Task<HealthyPerson> GetHealthyPerson(string imei)
        {
            using (IDbConnection connection = new MySqlConnection(healthy_conn))
            {
                string sql = @$"SELECT t1.imei,t2.person_id,t2.gender,t2.birth,t2.lifestyle,t2.medical_history
                           FROM healthy_person_device t1 
                           JOIN healthy_person t2 ON t1.person_id = t2.person_id
                           WHERE t1.imei = @imei AND t1.is_user_person = TRUE AND t1.is_deleted=FALSE";
                return await connection.QueryFirstOrDefaultAsync<HealthyPerson>(sql, new { imei });
            }
        }

    }
}
